@@ -36,7 +36,6 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.</small>
 
 #include <stdlib.h>
 #include <stdarg.h>
-#include <strings.h>
 #include <unicase.h>
 #include <unictype.h>
 #include <uniconv.h>
@@ -50,6 +49,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.</small>
 #endif
 
 static char localeCharSet_[64];
+static char consoleCharSet_[64];
 
 iLocalDef const char *currentLocaleLanguage_(void) {
     static iString *loc = NULL;
@@ -99,12 +99,18 @@ int width_Char(iChar d) {
     return uc_width(d, localeCharSet_);
 }
 
-void setLocaleCharSet_String(const char *charSet) {
+void setLocalEncoding_String(const char *encoding) {
     const size_t n = sizeof(localeCharSet_);
-    strncpy(localeCharSet_, charSet, n);
+    strncpy(localeCharSet_, encoding, n);
     localeCharSet_[n - 1] = 0;
 }
 
+void setConsoleEncoding_String(const char *encoding) {
+    const size_t n = sizeof(consoleCharSet_);
+    strncpy(consoleCharSet_, encoding, n);
+    consoleCharSet_[n - 1] = 0;
+}
+
 iString *new_String(void) {
     iString *d = iMalloc(String);
     init_String(d);
@@ -235,15 +241,24 @@ void initLocalCStr_String(iString *d, const char *localCStr) {
 }
 
 void initLocalCStrN_String(iString *d, const char *localCStr, size_t size) {
-    initBlockEncoding_String(d, &iBlockLiteral(localCStr, size, size), localeCharSet_);
+    initCStrNEncoding_String(d, localCStr, size, localeCharSet_);
 }
 
-void initBlockEncoding_String(iString *d, const iBlock *chars, const char *encoding) {
+static iBool isUtf8Encoding_(const char *encoding) {
+    return !encoding || !*encoding || !iCmpStrCase(encoding, "UTF-8") ||
+           !iCmpStrCase(encoding, "utf8");
+}
+
+void initCStrNEncoding_String(iString *d, const char *cstr, size_t n, const char *encoding) {
+    if (isUtf8Encoding_(encoding)) {
+        initCStrN_String(d, cstr, n);
+        return;
+    }
     size_t len = 0;
     uint8_t *data = u8_conv_from_encoding(encoding,
                                           iconveh_question_mark,
-                                          constData_Block(chars),
-                                          size_Block(chars),
+                                          cstr,
+                                          n,
                                           NULL,
                                           NULL,
                                           &len);
@@ -642,9 +657,12 @@ iChar last_String(const iString *d) {
     return iter.value;
 }
 
-iBlock *toLocal_String(const iString *d) {
+iBlock *toLocalCharSet_String(const iString *d, const char *charSet) {
+    if (isUtf8Encoding_(charSet)) {
+        return copy_Block(&d->chars);
+    }
     size_t len = 0;
-    char * str = u8_conv_to_encoding(localeCharSet_,
+    char * str = u8_conv_to_encoding(charSet,
                                      iconveh_question_mark,
                                      (const uint8_t *) cstr_String(d),
                                      size_String(d),
@@ -656,6 +674,14 @@ iBlock *toLocal_String(const iString *d) {
     return newPrealloc_Block(str, len, len + 1);
 }
 
+iBlock *toLocal_String(const iString *d) {
+    return toLocalCharSet_String(d, localeCharSet_);
+}
+
+iBlock *toConsole_String(const iString *d) {
+    return toLocalCharSet_String(d, consoleCharSet_);
+}
+
 iBlock *toUtf16_String(const iString *d) {
     size_t len = 0;
     uint16_t *u16 = u8_to_u16((const uint8_t *) cstr_String(d),
@@ -883,6 +909,14 @@ const char *cstr_Rangecc(iRangecc range) {
     return iCollectMem(copy);
 }
 
+const char *cstrLocal_Rangecc(iRangecc range) {
+    return cstr_Block(collect_Block(toLocal_Rangecc(range)));
+}
+
+const char *cstrConsole_Rangecc(iRangecc range) {
+    return cstr_Block(collect_Block(toConsole_Rangecc(range)));
+}
+
 const iString *string_Rangecc(iRangecc range) {
     return collect_String(newRange_String(range));
 }
@@ -913,20 +947,31 @@ int cmpCStrNSc_Rangecc(const iRangecc d, const char *cstr, size_t n, const iStri
     return cmp;
 }
 
-iBlock *toLocal_Rangecc(const iRangecc d) {
+iBlock *toLocalCharSet_Rangecc(const iRangecc d, const char *charSet) {
+    if (isUtf8Encoding_(charSet)) {
+        return newRange_Block(d);
+    }
     size_t len = 0;
-    char * str = u8_conv_to_encoding(localeCharSet_,
-                                    iconveh_question_mark,
-                                    (const uint8_t *) d.start,
-                                    size_Range(&d),
-                                    NULL,
-                                    NULL,
-                                    &len);
+    char * str = u8_conv_to_encoding(charSet,
+                                     iconveh_question_mark,
+                                     (const uint8_t *) d.start,
+                                     size_Range(&d),
+                                     NULL,
+                                     NULL,
+                                     &len);
     str = realloc(str, len + 1);
     str[len] = 0;
     return newPrealloc_Block(str, len, len + 1);
 }
 
+iBlock *toLocal_Rangecc(const iRangecc d) {
+    return toLocalCharSet_Rangecc(d, localeCharSet_);
+}
+
+iBlock *toConsole_Rangecc(const iRangecc d) {
+    return toLocalCharSet_Rangecc(d, consoleCharSet_);
+}
+
 iStringList *split_Rangecc(const iRangecc d, const char *separator) {
     iStringList *parts = new_StringList();
     iRangecc range = iNullRange;
@@ -1024,7 +1069,7 @@ iString *unquoteDelim_String(const iString *d, iChar delim) {
                     next_StringConstIterator(&i);
                     digits[j] = *i.pos;
                 }
-                uint16_t ch16[2] = { strtoul(digits, NULL, 16), 0 };
+                uint16_t ch16[2] = { (uint16_t)strtoul(digits, NULL, 16), 0 };
                 if (ch16[0] < 0xD800 || ch16[0] >= 0xE000) {
                     appendChar_String(unquot, ch16[0]);
                 }
@@ -1037,7 +1082,7 @@ iString *unquoteDelim_String(const iString *d, iChar delim) {
                         next_StringConstIterator(&i);
                         digits[j] = *i.pos;
                     }
-                    ch16[1] = strtoul(digits, NULL, 16);
+                    ch16[1] = (uint16_t)strtoul(digits, NULL, 16);
                     iString *u16 = newUtf16N_String(ch16, 2);
                     append_String(unquot, u16);
                     delete_String(u16);
@@ -1141,14 +1186,23 @@ int decodePrecedingBytes_MultibyteChar(const char *bytes, const char *start, iCh
     return (int) (bytes - precPos);
 }
 
+static tss_t tss_bufKey;
+
+static void bufKeyInit(void) {
+  tss_create(&tss_bufKey, free);
+}
+
 static char *threadLocalCharBuffer_(void) {
-    static tss_t bufKey = 0;
-    if (!bufKey) {
-        tss_create(&bufKey, free);
-    }
-    char *buf = tss_get(bufKey);
+#if defined(iHaveC11Threads)
+    static once_flag inited = ONCE_FLAG_INIT;
+#else
+    static once_flag inited = PTHREAD_ONCE_INIT;
+#endif
+    call_once(&inited, &bufKeyInit);
+
+    char *buf = tss_get(tss_bufKey);
     if (!buf) {
-        tss_set(bufKey, buf = malloc(iMultibyteCharMaxSize + 1));
+        tss_set(tss_bufKey, buf = malloc(iMultibyteCharMaxSize + 1));
     }
     return buf;
 }
@@ -1156,8 +1210,14 @@ static char *threadLocalCharBuffer_(void) {
 const char *cstrLocal_Char(iChar ch) {
     char *chBuf = threadLocalCharBuffer_();
     const iChar ucs[2] = { ch, 0 };
-    size_t len = iMultibyteCharMaxSize;
-    u32_conv_to_encoding(localeCharSet_, iconveh_question_mark, ucs, 1, NULL, chBuf, &len);
+    size_t      len    = iMultibyteCharMaxSize;
+    u32_conv_to_encoding(isUtf8Encoding_(localeCharSet_) ? "UTF-8" : localeCharSet_,
+                         iconveh_question_mark,
+                         ucs,
+                         1,
+                         NULL,
+                         chBuf,
+                         &len);
     chBuf[len] = 0;
     return chBuf;
 }
@@ -1250,7 +1310,11 @@ iStringComparison iCaseInsensitive = {
 };
 
 char *iDupStr(const char *a) {
+#if defined (iPlatformWindows)
+    return _strdup(a);
+#else
     return strdup(a);
+#endif
 }
 
 char *iStrStrN(const char *a, const char *b, size_t n) {
